@@ -1,8 +1,4 @@
-/*
- * -----------------------------------------------------------------
- * $Revision: 4956 $
- * $Date: 2016-09-23 11:15:59 -0700 (Fri, 23 Sep 2016) $
- * ----------------------------------------------------------------- 
+/* ----------------------------------------------------------------- 
  * Programmer(s): Radu Serban @ LLNL
  * -----------------------------------------------------------------
  * LLNS Copyright Start
@@ -35,16 +31,17 @@
  *
  * See D.B. Ozyurt and P.I. Barton, SISC 26(5) 1725-1743, 2005.
  *
- * -----------------------------------------------------------------
- */
+ * -----------------------------------------------------------------*/
 
 #include <stdio.h>
 #include <stdlib.h>
 
-#include <cvodes/cvodes.h>
-#include <cvodes/cvodes_dense.h>
-#include <nvector/nvector_serial.h>
-#include <sundials/sundials_math.h>
+#include <cvodes/cvodes.h>              /* prototypes for CVODES fcts., consts. */
+#include <nvector/nvector_serial.h>     /* access to serial N_Vector            */
+#include <sunmatrix/sunmatrix_dense.h>  /* access to band SUNMatrix             */
+#include <sunlinsol/sunlinsol_dense.h>  /* access to band SUNLinearSolver       */
+#include <cvodes/cvodes_direct.h>       /* access to CVDls interface            */
+#include <sundials/sundials_math.h>     /* definition of SUNRabs, SUNRexp, etc. */
 
 #define Ith(v,i)    NV_Ith_S(v,i-1)
 
@@ -82,6 +79,10 @@ static int fQB2(realtype t, N_Vector y, N_Vector *yS,
 void PrintFwdStats(void *cvode_mem);
 void PrintBckStats(void *cvode_mem, int idx);
 
+/* Private function to check function return values */
+
+static int check_flag(void *flagvalue, const char *funcname, int opt);
+
 /*
  *--------------------------------------------------------------------
  * MAIN PROGRAM
@@ -92,9 +93,11 @@ int main(int argc, char *argv[])
 {
   UserData data;
 
+  SUNMatrix A, AB1, AB2;
+  SUNLinearSolver LS, LSB1, LSB2;
   void *cvode_mem;
 
-  long int Neq, Np2;
+  sunindextype Neq, Np2;
   int Np;
 
   realtype t0, tf;
@@ -118,6 +121,14 @@ int main(int argc, char *argv[])
   realtype grdG_fwd[2], grdG_bck[2], grdG_cntr[2];
   realtype H11, H22;
 
+  data = NULL;
+  y = yQ = NULL;
+  yB1 = yB2 = NULL;
+  yQB1 = yQB2 = NULL;
+  A = AB1 = AB2 = NULL;
+  LS = LSB1 = LSB2 = NULL;
+  cvode_mem = NULL;
+
   /* User data structure */
 
   data = (UserData) malloc(sizeof *data);
@@ -144,46 +155,81 @@ int main(int argc, char *argv[])
   /* Initializations for forward problem */
 
   y = N_VNew_Serial(Neq);
+  if (check_flag((void *)y, "N_VNew_Serial", 0)) return(1);
   N_VConst(ONE, y);
 
   yQ = N_VNew_Serial(1);
+  if (check_flag((void *)yQ, "N_VNew_Serial", 0)) return(1);
   N_VConst(ZERO, yQ);
 
-  yS = N_VCloneVectorArray_Serial(Np, y);
+  yS = N_VCloneVectorArray(Np, y);
+  if (check_flag((void *)yS, "N_VCloneVectorArray", 0)) return(1);
   N_VConst(ZERO, yS[0]);
   N_VConst(ZERO, yS[1]);
 
-  yQS = N_VCloneVectorArray_Serial(Np, yQ);
+  yQS = N_VCloneVectorArray(Np, yQ);
+  if (check_flag((void *)yQS, "N_VCloneVectorArray", 0)) return(1);
   N_VConst(ZERO, yQS[0]);
   N_VConst(ZERO, yQS[1]);
 
   /* Create and initialize forward problem */
 
   cvode_mem = CVodeCreate(CV_BDF, CV_NEWTON);
+  if(check_flag((void *)cvode_mem, "CVodeCreate", 0)) return(1);
 
   flag = CVodeInit(cvode_mem, f, t0, y);
-  flag = CVodeSStolerances(cvode_mem, reltol, abstol);
-  flag = CVodeSetUserData(cvode_mem, data);
+  if(check_flag(&flag, "CVodeInit", 1)) return(1);
 
-  flag = CVDense(cvode_mem, Neq);
+  flag = CVodeSStolerances(cvode_mem, reltol, abstol);
+  if(check_flag(&flag, "CVodeSStolerances", 1)) return(1);
+
+  flag = CVodeSetUserData(cvode_mem, data);
+  if(check_flag(&flag, "CVodeSetUserData", 1)) return(1);
+
+  /* Create a dense SUNMatrix */
+  A = SUNDenseMatrix(Neq, Neq);
+  if(check_flag((void *)A, "SUNDenseMatrix", 0)) return(1);
+
+  /* Create banded SUNLinearSolver for the forward problem */
+  LS = SUNDenseLinearSolver(y, A);
+  if(check_flag((void *)LS, "SUNDenseLinearSolver", 0)) return(1);
+
+  /* Attach the matrix and linear solver */
+  flag = CVDlsSetLinearSolver(cvode_mem, LS, A);
+  if(check_flag(&flag, "CVDlsSetLinearSolver", 1)) return(1);
 
   flag = CVodeQuadInit(cvode_mem, fQ, yQ);
+  if(check_flag(&flag, "CVodeQuadInit", 1)) return(1);
+
   flag = CVodeQuadSStolerances(cvode_mem, reltol, abstolQ);
-  flag = CVodeSetQuadErrCon(cvode_mem, TRUE);
+  if(check_flag(&flag, "CVodeQuadSStolerances", 1)) return(1);
+
+  flag = CVodeSetQuadErrCon(cvode_mem, SUNTRUE);
+  if(check_flag(&flag, "CVodeSetQuadErrCon", 1)) return(1);
 
   flag = CVodeSensInit(cvode_mem, Np, CV_SIMULTANEOUS, fS, yS);
+  if(check_flag(&flag, "CVodeSensInit", 1)) return(1);
+
   flag = CVodeSensEEtolerances(cvode_mem);
-  flag = CVodeSetSensErrCon(cvode_mem, TRUE);
+  if(check_flag(&flag, "CVodeSensEEtolerances", 1)) return(1);
+
+  flag = CVodeSetSensErrCon(cvode_mem, SUNTRUE);
+  if(check_flag(&flag, "CVodeSetSensErrCon", 1)) return(1);
 
   flag = CVodeQuadSensInit(cvode_mem, fQS, yQS);
+  if(check_flag(&flag, "CVodeQuadSensInit", 1)) return(1);
 
   flag = CVodeQuadSensEEtolerances(cvode_mem);
-  flag = CVodeSetQuadSensErrCon(cvode_mem, TRUE);
+  if(check_flag(&flag, "CVodeQuadSensEEtolerances", 1)) return(1);
+
+  flag = CVodeSetQuadSensErrCon(cvode_mem, SUNTRUE);
+  if(check_flag(&flag, "CVodeSetQuadSensErrCon", 1)) return(1);
 
   /* Initialize ASA */
 
   steps = 100;
   flag = CVodeAdjInit(cvode_mem, steps, CV_POLYNOMIAL);
+  if(check_flag(&flag, "CVodeAdjInit", 1)) return(1);
 
   /* Forward integration */
 
@@ -192,16 +238,30 @@ int main(int argc, char *argv[])
   printf("-------------------\n\n");
 
   flag = CVodeF(cvode_mem, tf, y, &time, CV_NORMAL, &ncheck);
+  if(check_flag(&flag, "CVodeF", 1)) return(1);
 
   flag = CVodeGetQuad(cvode_mem, &time, yQ);
+  if(check_flag(&flag, "CVodeGetQuad", 1)) return(1);
+
   G = Ith(yQ,1);
 
   flag = CVodeGetSens(cvode_mem, &time, yS);
+  if(check_flag(&flag, "CVodeGetSens", 1)) return(1);
 
   flag = CVodeGetQuadSens(cvode_mem, &time, yQS);
+  if(check_flag(&flag, "CVodeGetQuadSens", 1)) return(1);
 
   printf("ncheck = %d\n", ncheck);
   printf("\n");
+#if defined(SUNDIALS_EXTENDED_PRECISION)
+  printf("     y:    %12.4Le %12.4Le %12.4Le", Ith(y,1), Ith(y,2), Ith(y,3));
+  printf("     G:    %12.4Le\n", Ith(yQ,1));
+  printf("\n");
+  printf("     yS1:  %12.4Le %12.4Le %12.4Le\n", Ith(yS[0],1), Ith(yS[0],2), Ith(yS[0],3));
+  printf("     yS2:  %12.4Le %12.4Le %12.4Le\n", Ith(yS[1],1), Ith(yS[1],2), Ith(yS[1],3));
+  printf("\n");
+  printf("   dG/dp:  %12.4Le %12.4Le\n", Ith(yQS[0],1), Ith(yQS[1],1));
+#else
   printf("     y:    %12.4e %12.4e %12.4e", Ith(y,1), Ith(y,2), Ith(y,3));
   printf("     G:    %12.4e\n", Ith(yQ,1));
   printf("\n");
@@ -209,6 +269,7 @@ int main(int argc, char *argv[])
   printf("     yS2:  %12.4e %12.4e %12.4e\n", Ith(yS[1],1), Ith(yS[1],2), Ith(yS[1],3));
   printf("\n");
   printf("   dG/dp:  %12.4e %12.4e\n", Ith(yQS[0],1), Ith(yQS[1],1));
+#endif
   printf("\n");
 
   printf("Final Statistics for forward pb.\n");
@@ -219,36 +280,96 @@ int main(int argc, char *argv[])
   /* Initializations for backward problems */
 
   yB1 = N_VNew_Serial(2*Neq);
+  if (check_flag((void *)yB1, "N_VNew_Serial", 0)) return(1);
   N_VConst(ZERO, yB1);
 
   yQB1 = N_VNew_Serial(Np2);
+  if (check_flag((void *)yQB1, "N_VNew_Serial", 0)) return(1);
   N_VConst(ZERO, yQB1);
 
   yB2 = N_VNew_Serial(2*Neq);
+  if (check_flag((void *)yB2, "N_VNew_Serial", 0)) return(1);
   N_VConst(ZERO, yB2);
 
   yQB2 = N_VNew_Serial(Np2);
+  if (check_flag((void *)yQB2, "N_VNew_Serial", 0)) return(1);
   N_VConst(ZERO, yQB2);
 
   /* Create and initialize backward problems (one for each column of the Hessian) */
 
+  /* -------------------------
+     First backward problem
+     -------------------------*/
+
   flag = CVodeCreateB(cvode_mem, CV_BDF, CV_NEWTON, &indexB1);
+  if(check_flag(&flag, "CVodeCreateB", 1)) return(1);
+
   flag = CVodeInitBS(cvode_mem, indexB1, fB1, tf, yB1);
+  if(check_flag(&flag, "CVodeInitBS", 1)) return(1);
+
   flag = CVodeSStolerancesB(cvode_mem, indexB1, reltol, abstolB);
+  if(check_flag(&flag, "CVodeSStolerancesB", 1)) return(1);
+
   flag = CVodeSetUserDataB(cvode_mem, indexB1, data);
+  if(check_flag(&flag, "CVodeSetUserDataB", 1)) return(1);
+
   flag = CVodeQuadInitBS(cvode_mem, indexB1, fQB1, yQB1);
+  if(check_flag(&flag, "CVodeQuadInitBS", 1)) return(1);
+
   flag = CVodeQuadSStolerancesB(cvode_mem, indexB1, reltol, abstolQB);
-  flag = CVodeSetQuadErrConB(cvode_mem, indexB1, TRUE);
-  flag = CVDenseB(cvode_mem, indexB1, 2*Neq);
+  if(check_flag(&flag, "CVodeQuadSStolerancesB", 1)) return(1);
+
+  flag = CVodeSetQuadErrConB(cvode_mem, indexB1, SUNTRUE);
+  if(check_flag(&flag, "CVodeSetQuadErrConB", 1)) return(1);
+
+  /* Create a dense SUNMatrix */
+  AB1 = SUNDenseMatrix(2*Neq, 2*Neq);
+  if(check_flag((void *)A, "SUNDenseMatrix", 0)) return(1);
+
+  /* Create banded SUNLinearSolver for the forward problem */
+  LSB1 = SUNDenseLinearSolver(yB1, AB1);
+  if(check_flag((void *)LSB1, "SUNDenseLinearSolver", 0)) return(1);
+
+  /* Attach the matrix and linear solver */
+  flag = CVDlsSetLinearSolverB(cvode_mem, indexB1, LSB1, AB1);
+  if(check_flag(&flag, "CVDlsSetLinearSolverB", 1)) return(1);
+
+  /* -------------------------
+     Second backward problem
+     -------------------------*/
 
   flag = CVodeCreateB(cvode_mem, CV_BDF, CV_NEWTON, &indexB2);
+  if(check_flag(&flag, "CVodeCreateB", 1)) return(1);
+
   flag = CVodeInitBS(cvode_mem, indexB2, fB2, tf, yB2);
+  if(check_flag(&flag, "CVodeInitBS", 1)) return(1);
+
   flag = CVodeSStolerancesB(cvode_mem, indexB2, reltol, abstolB);
+  if(check_flag(&flag, "CVodeSStolerancesB", 1)) return(1);
+
   flag = CVodeSetUserDataB(cvode_mem, indexB2, data);
+  if(check_flag(&flag, "CVodeSetUserDataB", 1)) return(1);
+
   flag = CVodeQuadInitBS(cvode_mem, indexB2, fQB2, yQB2);
+  if(check_flag(&flag, "CVodeQuadInitBS", 1)) return(1);
+
   flag = CVodeQuadSStolerancesB(cvode_mem, indexB2, reltol, abstolQB);
-  flag = CVodeSetQuadErrConB(cvode_mem, indexB2, TRUE);
-  flag = CVDenseB(cvode_mem, indexB2, 2*Neq);
+  if(check_flag(&flag, "CVodeQuadSStolerancesB", 1)) return(1);
+
+  flag = CVodeSetQuadErrConB(cvode_mem, indexB2, SUNTRUE);
+  if(check_flag(&flag, "CVodeSetQuadErrConB", 1)) return(1);
+
+  /* Create a dense SUNMatrix */
+  AB2 = SUNDenseMatrix(2*Neq, 2*Neq);
+  if(check_flag((void *)AB2, "SUNDenseMatrix", 0)) return(1);
+
+  /* Create banded SUNLinearSolver for the forward problem */
+  LSB2 = SUNDenseLinearSolver(yB2, AB2);
+  if(check_flag((void *)LSB2, "SUNDenseLinearSolver", 0)) return(1);
+
+  /* Attach the matrix and linear solver */
+  flag = CVDlsSetLinearSolverB(cvode_mem, indexB2, LSB2, AB2);
+  if(check_flag(&flag, "CVDlsSetLinearSolverB", 1)) return(1);
 
   /* Backward integration */
 
@@ -257,20 +378,37 @@ int main(int argc, char *argv[])
   printf("---------------------------------------------\n\n");
 
   flag = CVodeB(cvode_mem, t0, CV_NORMAL);
+  if(check_flag(&flag, "CVodeB", 1)) return(1);
 
   flag = CVodeGetB(cvode_mem, indexB1, &time, yB1);
+  if(check_flag(&flag, "CVodeGetB", 1)) return(1);
+
   flag = CVodeGetQuadB(cvode_mem, indexB1, &time, yQB1);
+  if(check_flag(&flag, "CVodeGetQuadB", 1)) return(1);
 
   flag = CVodeGetB(cvode_mem, indexB2, &time, yB2);
-  flag = CVodeGetQuadB(cvode_mem, indexB2, &time, yQB2);
+  if(check_flag(&flag, "CVodeGetB", 1)) return(1);
 
+  flag = CVodeGetQuadB(cvode_mem, indexB2, &time, yQB2);
+  if(check_flag(&flag, "CVodeGetQuadB", 1)) return(1);
+
+#if defined(SUNDIALS_EXTENDED_PRECISION)
+  printf("   dG/dp:  %12.4Le %12.4Le   (from backward pb. 1)\n", -Ith(yQB1,1), -Ith(yQB1,2));
+  printf("           %12.4Le %12.4Le   (from backward pb. 2)\n", -Ith(yQB2,1), -Ith(yQB2,2));
+#else
   printf("   dG/dp:  %12.4e %12.4e   (from backward pb. 1)\n", -Ith(yQB1,1), -Ith(yQB1,2));
   printf("           %12.4e %12.4e   (from backward pb. 2)\n", -Ith(yQB2,1), -Ith(yQB2,2));
+#endif  
   printf("\n");
   printf("   H = d2G/dp2:\n");
   printf("        (1)            (2)\n");
+#if defined(SUNDIALS_EXTENDED_PRECISION)
+  printf("  %12.4Le   %12.4Le\n", -Ith(yQB1,3) , -Ith(yQB2,3));
+  printf("  %12.4Le   %12.4Le\n", -Ith(yQB1,4) , -Ith(yQB2,4));
+#else
   printf("  %12.4e   %12.4e\n", -Ith(yQB1,3) , -Ith(yQB2,3));
   printf("  %12.4e   %12.4e\n", -Ith(yQB1,4) , -Ith(yQB2,4));
+#endif
   printf("\n");
 
   printf("Final Statistics for backward pb. 1\n");
@@ -281,9 +419,15 @@ int main(int argc, char *argv[])
   printf("-----------------------------------\n");
   PrintBckStats(cvode_mem, indexB2);
 
-  /* Free CVODES memory */
+  /* Free memory */
 
   CVodeFree(&cvode_mem);
+  SUNLinSolFree(LS);
+  SUNMatDestroy(A);
+  SUNLinSolFree(LSB1);
+  SUNMatDestroy(AB1);
+  SUNLinSolFree(LSB2);
+  SUNMatDestroy(AB2);
 
   /* Finite difference tests */
 
@@ -293,40 +437,86 @@ int main(int argc, char *argv[])
   printf("Finite Difference tests\n");
   printf("-----------------------\n\n");
 
+#if defined(SUNDIALS_EXTENDED_PRECISION)
+  printf("del_p = %Lg\n\n",dp);
+#else
   printf("del_p = %g\n\n",dp);
+#endif
 
   cvode_mem = CVodeCreate(CV_BDF, CV_NEWTON);
 
   N_VConst(ONE, y);
   N_VConst(ZERO, yQ);
+
   flag = CVodeInit(cvode_mem, f, t0, y);
+  if(check_flag(&flag, "CVodeInit", 1)) return(1);
+
   flag = CVodeSStolerances(cvode_mem, reltol, abstol);
+  if(check_flag(&flag, "CVodeSStolerances", 1)) return(1);
+
   flag = CVodeSetUserData(cvode_mem, data);
-  flag = CVDense(cvode_mem, Neq);
+  if(check_flag(&flag, "CVodeSetUserData", 1)) return(1);
+
+  /* Create a dense SUNMatrix */
+  A = SUNDenseMatrix(Neq, Neq);
+  if(check_flag((void *)A, "SUNDenseMatrix", 0)) return(1);
+
+  /* Create banded SUNLinearSolver for the forward problem */
+  LS = SUNDenseLinearSolver(y, A);
+  if(check_flag((void *)LS, "SUNDenseLinearSolver", 0)) return(1);
+
+  /* Attach the matrix and linear solver */
+  flag = CVDlsSetLinearSolver(cvode_mem, LS, A);
+  if(check_flag(&flag, "CVDlsSetLinearSolver", 1)) return(1);
+
   flag = CVodeQuadInit(cvode_mem, fQ, yQ);
+  if(check_flag(&flag, "CVodeQuadInit", 1)) return(1);
+
   flag = CVodeQuadSStolerances(cvode_mem, reltol, abstolQ);
-  flag = CVodeSetQuadErrCon(cvode_mem, TRUE);
+  if(check_flag(&flag, "CVodeQuadSStolerances", 1)) return(1);
+
+  flag = CVodeSetQuadErrCon(cvode_mem, SUNTRUE);
+  if(check_flag(&flag, "CVodeSetQuadErrCon", 1)) return(1);
 
   data->p1 += dp;
 
   flag = CVode(cvode_mem, tf, y, &time, CV_NORMAL);
+  if(check_flag(&flag, "CVode", 1)) return(1);
+
   flag = CVodeGetQuad(cvode_mem, &time, yQ);
+  if(check_flag(&flag, "CVodeGetQuad", 1)) return(1);
+
   Gp = Ith(yQ,1);
+
+#if defined(SUNDIALS_EXTENDED_PRECISION)
+  printf("p1+  y:   %12.4Le %12.4Le %12.4Le", Ith(y,1), Ith(y,2), Ith(y,3));
+  printf("     G:   %12.4Le\n",Ith(yQ,1));
+#else
   printf("p1+  y:   %12.4e %12.4e %12.4e", Ith(y,1), Ith(y,2), Ith(y,3));
   printf("     G:   %12.4e\n",Ith(yQ,1));
-
+#endif  
   data->p1 -= 2.0*dp;
 
   N_VConst(ONE, y);
   N_VConst(ZERO, yQ);
+
   CVodeReInit(cvode_mem, t0, y);
   CVodeQuadReInit(cvode_mem, yQ);
+
   flag = CVode(cvode_mem, tf, y, &time, CV_NORMAL);
+  if(check_flag(&flag, "CVode", 1)) return(1);
+
   flag = CVodeGetQuad(cvode_mem, &time, yQ);
+  if(check_flag(&flag, "CVodeGetQuad", 1)) return(1);
+
   Gm = Ith(yQ,1);
+#if defined(SUNDIALS_EXTENDED_PRECISION)
+  printf("p1-  y:   %12.4Le %12.4Le %12.4Le", Ith(y,1), Ith(y,2), Ith(y,3));
+  printf("     G:   %12.4Le\n",Ith(yQ,1));
+#else
   printf("p1-  y:   %12.4e %12.4e %12.4e", Ith(y,1), Ith(y,2), Ith(y,3));
   printf("     G:   %12.4e\n",Ith(yQ,1));
- 
+#endif
   data->p1 += dp;
 
   grdG_fwd[0] = (Gp-G)/dp;
@@ -338,26 +528,46 @@ int main(int argc, char *argv[])
 
   N_VConst(ONE, y);
   N_VConst(ZERO, yQ);
+
   CVodeReInit(cvode_mem, t0, y);
   CVodeQuadReInit(cvode_mem, yQ);
+
   flag = CVode(cvode_mem, tf, y, &time, CV_NORMAL);
+  if(check_flag(&flag, "CVode", 1)) return(1);
+
   flag = CVodeGetQuad(cvode_mem, &time, yQ);
+  if(check_flag(&flag, "CVodeGetQuad", 1)) return(1);
+
   Gp = Ith(yQ,1);
+#if defined(SUNDIALS_EXTENDED_PRECISION)
+  printf("p2+  y:   %12.4Le %12.4Le %12.4Le", Ith(y,1), Ith(y,2), Ith(y,3));
+  printf("     G:   %12.4Le\n",Ith(yQ,1));
+#else
   printf("p2+  y:   %12.4e %12.4e %12.4e", Ith(y,1), Ith(y,2), Ith(y,3));
   printf("     G:   %12.4e\n",Ith(yQ,1));
- 
+#endif
   data->p2 -= 2.0*dp;
 
   N_VConst(ONE, y);
   N_VConst(ZERO, yQ);
+
   CVodeReInit(cvode_mem, t0, y);
   CVodeQuadReInit(cvode_mem, yQ);
+
   flag = CVode(cvode_mem, tf, y, &time, CV_NORMAL);
+  if(check_flag(&flag, "CVode", 1)) return(1);
+
   flag = CVodeGetQuad(cvode_mem, &time, yQ);
+  if(check_flag(&flag, "CVodeGetQuad", 1)) return(1);
+
   Gm = Ith(yQ,1);
+#if defined(SUNDIALS_EXTENDED_PRECISION)
+  printf("p2-  y:   %12.4Le %12.4Le %12.4Le", Ith(y,1), Ith(y,2), Ith(y,3));
+  printf("     G:   %12.4Le\n",Ith(yQ,1));
+#else
   printf("p2-  y:   %12.4e %12.4e %12.4e", Ith(y,1), Ith(y,2), Ith(y,3));
   printf("     G:   %12.4e\n",Ith(yQ,1));
-
+#endif  
   data->p2 += dp;
 
   grdG_fwd[1] = (Gp-G)/dp;
@@ -367,28 +577,38 @@ int main(int argc, char *argv[])
 
   printf("\n");
 
+#if defined(SUNDIALS_EXTENDED_PRECISION)
+  printf("   dG/dp:  %12.4Le %12.4Le   (fwd FD)\n", grdG_fwd[0], grdG_fwd[1]);
+  printf("           %12.4Le %12.4Le   (bck FD)\n", grdG_bck[0], grdG_bck[1]);
+  printf("           %12.4Le %12.4Le   (cntr FD)\n", grdG_cntr[0], grdG_cntr[1]);
+  printf("\n");
+  printf("  H(1,1):  %12.4Le\n", H11);
+  printf("  H(2,2):  %12.4Le\n", H22);
+#else
   printf("   dG/dp:  %12.4e %12.4e   (fwd FD)\n", grdG_fwd[0], grdG_fwd[1]);
   printf("           %12.4e %12.4e   (bck FD)\n", grdG_bck[0], grdG_bck[1]);
   printf("           %12.4e %12.4e   (cntr FD)\n", grdG_cntr[0], grdG_cntr[1]);
   printf("\n");
   printf("  H(1,1):  %12.4e\n", H11);
   printf("  H(2,2):  %12.4e\n", H22);
-
+#endif  
 
   /* Free memory */
 
   CVodeFree(&cvode_mem);
+  SUNLinSolFree(LS);
+  SUNMatDestroy(A);
 
-  N_VDestroy_Serial(y);
-  N_VDestroy_Serial(yQ);
+  N_VDestroy(y);
+  N_VDestroy(yQ);
 
-  N_VDestroyVectorArray_Serial(yS, Np);
-  N_VDestroyVectorArray_Serial(yQS, Np);
+  N_VDestroyVectorArray(yS, Np);
+  N_VDestroyVectorArray(yQS, Np);
 
-  N_VDestroy_Serial(yB1);
-  N_VDestroy_Serial(yQB1);
-  N_VDestroy_Serial(yB2);
-  N_VDestroy_Serial(yQB2);
+  N_VDestroy(yB1);
+  N_VDestroy(yQB1);
+  N_VDestroy(yB2);
+  N_VDestroy(yQB2);
 
   free(data);
 
@@ -782,3 +1002,41 @@ void PrintBckStats(void *cvode_mem, int idx)
 
 
 }
+
+/*
+ * Check function return value...
+ *   opt == 0 means SUNDIALS function allocates memory so check if
+ *            returned NULL pointer
+ *   opt == 1 means SUNDIALS function returns a flag so check if
+ *            flag >= 0
+ *   opt == 2 means function allocates memory so check if returned
+ *            NULL pointer 
+ */
+
+static int check_flag(void *flagvalue, const char *funcname, int opt)
+{
+  int *errflag;
+
+  /* Check if SUNDIALS function returned NULL pointer - no memory allocated */
+  if (opt == 0 && flagvalue == NULL) {
+    fprintf(stderr, "\nSUNDIALS_ERROR: %s() failed - returned NULL pointer\n\n",
+	    funcname);
+    return(1); }
+
+  /* Check if flag < 0 */
+  else if (opt == 1) {
+    errflag = (int *) flagvalue;
+    if (*errflag < 0) {
+      fprintf(stderr, "\nSUNDIALS_ERROR: %s() failed with flag = %d\n\n",
+	      funcname, *errflag);
+      return(1); }}
+
+  /* Check if function returned NULL pointer - no memory allocated */
+  else if (opt == 2 && flagvalue == NULL) {
+    fprintf(stderr, "\nMEMORY_ERROR: %s() failed - returned NULL pointer\n\n",
+	    funcname);
+    return(1); }
+
+  return(0);
+}
+
