@@ -1,8 +1,4 @@
-/*
- * -----------------------------------------------------------------
- * $Revision: 4834 $
- * $Date: 2016-08-01 16:59:05 -0700 (Mon, 01 Aug 2016) $
- * -----------------------------------------------------------------
+/* -----------------------------------------------------------------
  * Programmer(s): Cosmin Petra and Radu Serban @ LLNL
  * -----------------------------------------------------------------
  * Example problem:
@@ -25,7 +21,7 @@
  * user routine fS (of type SensRhs1Fn).
  * Any of two sensitivity methods (SIMULTANEOUS and STAGGERED can be
  * used and sensitivities may be included in the error test or not 
- *(error control set on TRUE or FALSE, respectively).
+ *(error control set on SUNTRUE or SUNFALSE, respectively).
  *
  * Execution:
  *
@@ -35,23 +31,23 @@
  *    % idasRoberts_FSA_dns -sensi sensi_meth err_con
  * where sensi_meth is one of {sim, stg} and err_con is one of
  * {t, f}.
- * -----------------------------------------------------------------
- */
+ * -----------------------------------------------------------------*/
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
-#include <idas/idas.h>           /* prototypes for IDAS fcts. and consts. */
-#include <idas/idas_dense.h>
-#include <nvector/nvector_serial.h>  /* defs. of serial NVECTOR fcts. and macros  */
-#include <sundials/sundials_types.h> /* def. of type realtype */
-#include <sundials/sundials_math.h>  /* definition of ABS */
+#include <idas/idas.h>                 /* prototypes for IDA fcts., consts.    */
+#include <nvector/nvector_serial.h>    /* access to serial N_Vector            */
+#include <sunmatrix/sunmatrix_dense.h> /* access to dense SUNMatrix            */
+#include <sunlinsol/sunlinsol_dense.h> /* access to dense SUNLinearSolver      */
+#include <idas/idas_direct.h>          /* access to IDADls interface           */
+#include <sundials/sundials_types.h>   /* defs. of realtype, sunindextype      */
+#include <sundials/sundials_math.h>    /* defs. of SUNRabs, SUNRexp, etc.      */
 
 /* Accessor macros */
 
 #define Ith(v,i)    NV_Ith_S(v,i-1)       /* i-th vector component i=1..NEQ */
-#define IJth(A,i,j) DENSE_ELEM(A,i-1,j-1) /* (i,j)-th matrix component i,j=1..NEQ */
 
 /* Problem Constants */
 
@@ -112,6 +108,8 @@ static int check_flag(void *flagvalue, const char *funcname, int opt);
 int main(int argc, char *argv[])
 {
   void *ida_mem;
+  SUNMatrix A;
+  SUNLinearSolver LS;
   UserData data;
   realtype reltol, t, tout;
   N_Vector y, yp, abstol, id;
@@ -126,8 +124,10 @@ int main(int argc, char *argv[])
   N_Vector yQ, *yQS;
 
   ida_mem = NULL;
+  A       = NULL;
+  LS      = NULL;
   data    = NULL;
-  y       =  NULL;
+  y       = NULL;
   yS      = NULL;
   ypS     = NULL;
 
@@ -187,9 +187,17 @@ int main(int argc, char *argv[])
   flag = IDASetUserData(ida_mem, data);
   if (check_flag(&flag, "IDASetUserData", 1)) return(1);
 
-  /* Attach linear solver */
-  flag = IDADense(ida_mem, NEQ);
-  if (check_flag(&flag, "IDADense", 1)) return(1);
+  /* Create dense SUNMatrix for use in linear solves */
+  A = SUNDenseMatrix(NEQ, NEQ);
+  if(check_flag((void *)A, "SUNDenseMatrix", 0)) return(1);
+
+  /* Create dense SUNLinearSolver object */
+  LS = SUNDenseLinearSolver(y, A);
+  if(check_flag((void *)LS, "SUNDenseLinearSolver", 0)) return(1);
+
+  /* Attach the matrix and linear solver */
+  flag = IDADlsSetLinearSolver(ida_mem, LS, A);
+  if(check_flag(&flag, "IDADlsSetLinearSolver", 1)) return(1);
 
   printf("\n3-species chemical kinetics problem\n");
 
@@ -200,12 +208,12 @@ int main(int argc, char *argv[])
     pbar[1] = data->p[1];
     pbar[2] = data->p[2];
 
-    yS = N_VCloneVectorArray_Serial(NS, y);
-    if (check_flag((void *)yS, "N_VCloneVectorArray_Serial", 0)) return(1);
+    yS = N_VCloneVectorArray(NS, y);
+    if (check_flag((void *)yS, "N_VCloneVectorArray", 0)) return(1);
     for (is=0;is<NS;is++) N_VConst(ZERO, yS[is]);
     
-    ypS = N_VCloneVectorArray_Serial(NS, y);
-    if (check_flag((void *)ypS, "N_VCloneVectorArray_Serial", 0)) return(1);
+    ypS = N_VCloneVectorArray(NS, y);
+    if (check_flag((void *)ypS, "N_VCloneVectorArray", 0)) return(1);
     for (is=0;is<NS;is++) N_VConst(ZERO, ypS[is]);
 
     /* 
@@ -252,10 +260,12 @@ int main(int argc, char *argv[])
 
   IDAQuadInit(ida_mem, rhsQ, yQ);
 
-  yQS = N_VCloneVectorArray_Serial(NS, yQ);
-  for (is=0;is<NS;is++) N_VConst(ZERO, yQS[is]);  
+  if (sensi) {
+    yQS = N_VCloneVectorArray(NS, yQ);
+    for (is=0;is<NS;is++) N_VConst(ZERO, yQS[is]);  
 
-  IDAQuadSensInit(ida_mem, NULL, yQS);
+    IDAQuadSensInit(ida_mem, NULL, yQS);
+  }
 
   /* Call IDACalcIC to compute consistent initial conditions. If sensitivity is
      enabled, this function also try to find consistent IC for the sensitivities. */
@@ -302,27 +312,45 @@ int main(int argc, char *argv[])
 
   printf("\nQuadrature:\n");
   IDAGetQuad(ida_mem, &t, yQ);
-  printf("G:      %10.4e\n", Ith(yQ,1)); 
+#if defined(SUNDIALS_EXTENDED_PRECISION)
+  printf("G:      %10.4Le\n", Ith(yQ,1)); 
+#else
+  printf("G:      %10.4e\n", Ith(yQ,1));
+#endif
 
   if(sensi) {
     IDAGetQuadSens(ida_mem, &t, yQS);
+#if defined(SUNDIALS_EXTENDED_PRECISION)
+    printf("\nSensitivities at t=%Lg:\n",t);
+    printf("dG/dp1: %11.4Le\n", Ith(yQS[0], 1));
+    printf("dG/dp1: %11.4Le\n", Ith(yQS[1], 1));
+    printf("dG/dp1: %11.4Le\n", Ith(yQS[2], 1));
+#else
     printf("\nSensitivities at t=%g:\n",t);
     printf("dG/dp1: %11.4e\n", Ith(yQS[0], 1));
     printf("dG/dp1: %11.4e\n", Ith(yQS[1], 1));
     printf("dG/dp1: %11.4e\n", Ith(yQS[2], 1));
+#endif
   }
 
   /* Print final statistics */
   PrintFinalStats(ida_mem, sensi);
 
   /* Free memory */
-  N_VDestroy_Serial(y);
+  N_VDestroy(y);
+  N_VDestroy(yp);
+  N_VDestroy(abstol);
+  N_VDestroy(id);
+  N_VDestroy(yQ);
   if (sensi) {
-    N_VDestroyVectorArray_Serial(yS, NS);
+    N_VDestroyVectorArray(yS,  NS);
+    N_VDestroyVectorArray(ypS, NS);
+    N_VDestroyVectorArray(yQS, NS);
   }
   free(data);
   IDAFree(&ida_mem);
-  N_VDestroy_Serial(yQ);
+  SUNLinSolFree(LS);
+  SUNMatDestroy(A);
 
   return(0);
 }
@@ -463,16 +491,16 @@ static int rhsQ(realtype t, N_Vector y, N_Vector yp,
 static void ProcessArgs(int argc, char *argv[], 
                         booleantype *sensi, int *sensi_meth, booleantype *err_con)
 {
-  *sensi = FALSE;
+  *sensi = SUNFALSE;
   *sensi_meth = -1;
-  *err_con = FALSE;
+  *err_con = SUNFALSE;
 
   if (argc < 2) WrongArgs(argv[0]);
 
   if (strcmp(argv[1],"-nosensi") == 0)
-    *sensi = FALSE;
+    *sensi = SUNFALSE;
   else if (strcmp(argv[1],"-sensi") == 0)
-    *sensi = TRUE;
+    *sensi = SUNTRUE;
   else
     WrongArgs(argv[0]);
   
@@ -489,9 +517,9 @@ static void ProcessArgs(int argc, char *argv[],
       WrongArgs(argv[0]);
 
     if (strcmp(argv[3],"t") == 0)
-      *err_con = TRUE;
+      *err_con = SUNTRUE;
     else if (strcmp(argv[3],"f") == 0)
-      *err_con = FALSE;
+      *err_con = SUNFALSE;
     else
       WrongArgs(argv[0]);
   }
@@ -512,7 +540,7 @@ static void PrintIC(N_Vector y, N_Vector yp)
 {
   realtype* data;
 
-  data = N_VGetArrayPointer_Serial(y);
+  data = N_VGetArrayPointer(y);
   printf("\n\nConsistent IC:\n");
   printf("\ty = ");
 #if defined(SUNDIALS_EXTENDED_PRECISION)
@@ -523,7 +551,7 @@ static void PrintIC(N_Vector y, N_Vector yp)
   printf("%12.4e %12.4e %12.4e \n", data[0], data[1], data[2]);
 #endif
 
-  data = N_VGetArrayPointer_Serial(yp);
+  data = N_VGetArrayPointer(yp);
   printf("\typ= ");
 #if defined(SUNDIALS_EXTENDED_PRECISION)
   printf("%12.4Le %12.4Le %12.4Le \n", data[0], data[1], data[2]);
@@ -538,7 +566,7 @@ static void PrintSensIC(N_Vector y, N_Vector yp, N_Vector* yS, N_Vector* ypS)
 {
   realtype *sdata;
 
-  sdata = N_VGetArrayPointer_Serial(yS[0]);
+  sdata = N_VGetArrayPointer(yS[0]);
   printf("                  Sensitivity 1  ");
 
   printf("\n\ts1 = ");
@@ -549,7 +577,7 @@ static void PrintSensIC(N_Vector y, N_Vector yp, N_Vector* yS, N_Vector* ypS)
 #else
   printf("%12.4e %12.4e %12.4e \n", sdata[0], sdata[1], sdata[2]);
 #endif
-  sdata = N_VGetArrayPointer_Serial(ypS[0]);
+  sdata = N_VGetArrayPointer(ypS[0]);
   printf("\ts1'= ");
 #if defined(SUNDIALS_EXTENDED_PRECISION)
   printf("%12.4Le %12.4Le %12.4Le \n", sdata[0], sdata[1], sdata[2]);
@@ -561,7 +589,7 @@ static void PrintSensIC(N_Vector y, N_Vector yp, N_Vector* yS, N_Vector* ypS)
 
 
   printf("                  Sensitivity 2  ");
-  sdata = N_VGetArrayPointer_Serial(yS[1]);
+  sdata = N_VGetArrayPointer(yS[1]);
   printf("\n\ts2 = ");
 #if defined(SUNDIALS_EXTENDED_PRECISION)
   printf("%12.4Le %12.4Le %12.4Le \n", sdata[0], sdata[1], sdata[2]);
@@ -570,7 +598,7 @@ static void PrintSensIC(N_Vector y, N_Vector yp, N_Vector* yS, N_Vector* ypS)
 #else
   printf("%12.4e %12.4e %12.4e \n", sdata[0], sdata[1], sdata[2]);
 #endif
-  sdata = N_VGetArrayPointer_Serial(ypS[1]);
+  sdata = N_VGetArrayPointer(ypS[1]);
   printf("\ts2'= ");
 #if defined(SUNDIALS_EXTENDED_PRECISION)
   printf("%12.4Le %12.4Le %12.4Le \n", sdata[0], sdata[1], sdata[2]);
@@ -582,7 +610,7 @@ static void PrintSensIC(N_Vector y, N_Vector yp, N_Vector* yS, N_Vector* ypS)
 
 
   printf("                  Sensitivity 3  ");
-  sdata = N_VGetArrayPointer_Serial(yS[2]);
+  sdata = N_VGetArrayPointer(yS[2]);
   printf("\n\ts3 = ");
 #if defined(SUNDIALS_EXTENDED_PRECISION)
   printf("%12.4Le %12.4Le %12.4Le \n", sdata[0], sdata[1], sdata[2]);
@@ -591,7 +619,7 @@ static void PrintSensIC(N_Vector y, N_Vector yp, N_Vector* yS, N_Vector* ypS)
 #else
   printf("%12.4e %12.4e %12.4e \n", sdata[0], sdata[1], sdata[2]);
 #endif
-  sdata = N_VGetArrayPointer_Serial(ypS[2]);
+  sdata = N_VGetArrayPointer(ypS[2]);
   printf("\ts3'= ");
 #if defined(SUNDIALS_EXTENDED_PRECISION)
   printf("%12.4Le %12.4Le %12.4Le \n", sdata[0], sdata[1], sdata[2]);
@@ -614,7 +642,7 @@ static void PrintOutput(void *ida_mem, realtype t, N_Vector u)
   int qu, flag;
   realtype hu, *udata;
   
-  udata = N_VGetArrayPointer_Serial(u);
+  udata = N_VGetArrayPointer(u);
 
   flag = IDAGetNumSteps(ida_mem, &nst);
   check_flag(&flag, "IDAGetNumSteps", 1);
@@ -651,7 +679,7 @@ static void PrintSensOutput(N_Vector *uS)
 {
   realtype *sdata;
 
-  sdata = N_VGetArrayPointer_Serial(uS[0]);
+  sdata = N_VGetArrayPointer(uS[0]);
   printf("                  Sensitivity 1  ");
 
 #if defined(SUNDIALS_EXTENDED_PRECISION)
@@ -662,7 +690,7 @@ static void PrintSensOutput(N_Vector *uS)
   printf("%12.4e %12.4e %12.4e \n", sdata[0], sdata[1], sdata[2]);
 #endif
   
-  sdata = N_VGetArrayPointer_Serial(uS[1]);
+  sdata = N_VGetArrayPointer(uS[1]);
   printf("                  Sensitivity 2  ");
 
 #if defined(SUNDIALS_EXTENDED_PRECISION)
@@ -673,7 +701,7 @@ static void PrintSensOutput(N_Vector *uS)
   printf("%12.4e %12.4e %12.4e \n", sdata[0], sdata[1], sdata[2]);
 #endif
 
-  sdata = N_VGetArrayPointer_Serial(uS[2]);
+  sdata = N_VGetArrayPointer(uS[2]);
   printf("                  Sensitivity 3  ");
 
 #if defined(SUNDIALS_EXTENDED_PRECISION)
